package Apache::AuthDigest::API::Full;

use Apache::Log;

use Apache::AuthDigest::API;

use 5.006;

use strict;

our $VERSION = '0.01';
our @ISA = qw(Apache::AuthDigest::API);

__END__

=head1 NAME

Apache::AuthDigest::API::Full - RFC 2617 complaint Digest API

=head1 DESCRIPTION

This class is currently unimplementated

=head1 SEE ALSO

perl(1), mod_perl(1), Apache(3), Apache::AuthDigest::API

=head1 AUTHORS

Geoffrey Young E<lt>geoff@modperlcookbook.orgE<gt>

Paul Lindner E<lt>paul@modperlcookbook.orgE<gt>

Randy Kobes E<lt>randy@modperlcookbook.orgE<gt>

=head1 COPYRIGHT

Copyright (c) 2002, Geoffrey Young, Paul Lindner, Randy Kobes.  

All rights reserved.

This module is free software.  It may be used, redistributed
and/or modified under the same terms as Perl itself.

=head1 HISTORY

This code is derived from the I<Cookbook::DigestAPI> module,
available as part of "The mod_perl Developer's Cookbook".

For more information, visit http://www.modperlcookbook.org/

=cut
